#!/usr/bin/env bash
set -euo pipefail

IFS=$'\n'

lock_file="/tmp/mosaic-configured"

if [ -e "${lock_file}" ]; then
  # Already configured.
  exit
fi

function get_digest_of_env_variable {
  local env_var="$1"

  if [ -n "${!env_var:-}" ]; then
    printf "Variable %s digest - " "${env_var}"
    printf '%s' "${!env_var:-}" | md5sum | awk '{ print $1 }'
  fi

}

# Variables that get set in CI/CD we want to know when they change
SECRET_VALUE_VARS=(
  MOSAIC_SSH_PRIVATE_KEY
  MOSAIC_HELM_KEY
  MOSAIC_HELM_CERT
  MOSAIC_HELM_CA_CERT
  MOSAIC_KUBERNETES_CONFIG
  DEV_AWS_ACCESS_KEY_ID
  DEV_AWS_SECRET_ACCESS_KEY
  TEST_AWS_ACCESS_KEY_ID
  TEST_AWS_SECRET_ACCESS_KEY
  PROD_AWS_ACCESS_KEY_ID
  PROD_AWS_SECRET_ACCESS_KEY
)

for VAR in "${SECRET_VALUE_VARS[@]}"
do
  get_digest_of_env_variable "$VAR"
done

function configured {
  local check="$1"
  local service="${check%% *}"
  tmp=$(mktemp)

  if eval "${check}" > /dev/null 2>"${tmp}"; then
    echo "Configured: ${service}"
  else
    echo "Unconfigured: ${service}"
    cat "${tmp}"
  fi
}

echo "Checking if the creds are Role creds"

CURRENT_ROLE_ARN=$(aws sts get-caller-identity | jq -r .Arn)
CICD_ROLE="arn:aws:iam::617762925086:role/${PRODUCT}-${APPLICATION}-${ENV_CATEGORY}-cicd"

# If we are already running as a role - no need to do anything
if [[ $CURRENT_ROLE_ARN =~ arn:aws:sts::617762925086:(assumed-role|role)\/(.*) ]]; then
  echo "AWS Role"
  aws sts get-caller-identity | jq -r .Arn || true
else
  echo "AWS User"
  configured "aws sts get-caller-identity"

  echo "CICD Role - ${CICD_ROLE}"

  # TODO Add some interesting information about what's running to this session name?
  ROLE_SESSION_NAME="mosaic-deploy-scripts"
  profile_name=default

  temp_role=$(aws sts assume-role \
              --role-arn "$CICD_ROLE" \
              --role-session-name "$ROLE_SESSION_NAME")

  # Write creds out to default profile
  AWS_ACCESS_KEY_ID=$(echo "$temp_role" | jq -r .Credentials.AccessKeyId)
  AWS_SECRET_ACCESS_KEY=$(echo "$temp_role" | jq -r .Credentials.SecretAccessKey)
  AWS_SESSION_TOKEN=$(echo "$temp_role" | jq -r .Credentials.SessionToken)

  aws configure set aws_access_key_id "$AWS_ACCESS_KEY_ID" --profile "$profile_name"
  aws configure set aws_secret_access_key "$AWS_SECRET_ACCESS_KEY" --profile "$profile_name"
  aws configure set aws_session_token "$AWS_SESSION_TOKEN" --profile "$profile_name"

  # Ignore any ENV variables for AWS Auth and use default profile with the assume CICD Role
  unset AWS_ACCESS_KEY_ID
  unset AWS_SECRET_ACCESS_KEY
  unset AWS_SESSION_TOKEN

  echo "AWS Role"
  aws sts get-caller-identity | jq -r .Arn || true
fi

configured "aws sts get-caller-identity"

if [ -n "${MOSAIC_SSH_PRIVATE_KEY-}" ]; then
  mkdir -p ~/.ssh
  chmod 700 ~/.ssh
  cat <<< "${MOSAIC_SSH_PRIVATE_KEY}" > ~/.ssh/id_rsa
  chmod 600 ~/.ssh/id_rsa
elif [ -n "${CI_JOB_TOKEN}" ]; then
  echo -e "machine gitlab.bit9.local\nlogin gitlab-ci-token\npassword ${CI_JOB_TOKEN}" > ~/.netrc
  git config --global url.https://gitlab.bit9.local/.insteadOf ssh://git@gitlab.bit9.local/
  echo "Configured: gitlab-ci-token"
fi

configured "ssh-keygen -lvf ~/.ssh/id_rsa"

mkdir -p ~/.kube

# Check if kube.config object exists in S3
kube_config_in_s3=0

# Ensure ENV_CATEGORY and REGION is set
if [[ -z ${ENV_CATEGORY-} || -z ${REGION-} ]]; then
  echo "ENV_CATEGORY and REGION env variables must be set to pull kubeconfig from S3"
  kube_config_in_s3=1
else
  eval aws s3 ls "s3://cb-mosaic-deploy/${PRODUCT}/${APPLICATION}/${ENV_CATEGORY}/${REGION}/kube.config" > /dev/null || kube_config_in_s3=$?
fi

# Check for existence of the kube.config in s3
if [ $kube_config_in_s3 == 0 ]; then
  echo "Setting up kube config using S3 kube.config object"
  aws s3 cp "s3://cb-mosaic-deploy/${PRODUCT}/${APPLICATION}/${ENV_CATEGORY}/${REGION}/kube.config" ~/.kube/config
elif [ $kube_config_in_s3 == 1 ]; then
  echo "kube.config does not exist in s3 - falling back to MOSAIC_KUBERNETES_CONFIG"
  if [ -n "${MOSAIC_KUBERNETES_CONFIG-}" ]; then
    echo "Setting up kube config using CI/CD env vars"
    echo "WARNING Pulling kube config from CI/CD will be deprecated" >&2
    cat <<< "${MOSAIC_KUBERNETES_CONFIG}" > ~/.kube/config
  fi
else
  echo "kube.config exists in s3 check return unknown error code ${kube_config_in_s3}"
  exit 1
fi


configured "kubectl version"

if [ -n "${MOSAIC_HELM_KEY-}" ]; then
  echo "Setting up Helm using CI/CD env vars"
  mkdir -p ~/.helm
  cat <<< "${MOSAIC_HELM_KEY}" > "$(helm home)"/key.pem
  cat <<< "${MOSAIC_HELM_CERT}" > "$(helm home)"/cert.pem
  cat <<< "${MOSAIC_HELM_CA_CERT}" > "$(helm home)"/ca.pem
elif [ "${ENABLE_HELM3:=false}" == "true" ]; then
  echo "Helm 3 no Tiller! :-)"
elif [ -n "${ENV_CATEGORY-}" ]; then
  echo "Setting up Helm using AWS CLI"
  aws s3 cp "s3://cb-mosaic-deploy/${PRODUCT}/${APPLICATION}/${ENV_CATEGORY}/${REGION}/tiller/helm.key.pem" "$(helm home)/key.pem"
  aws s3 cp "s3://cb-mosaic-deploy/${PRODUCT}/${APPLICATION}/${ENV_CATEGORY}/${REGION}/tiller/helm.cert.pem" "$(helm home)/cert.pem"
  aws s3 cp "s3://cb-mosaic-deploy/${PRODUCT}/${APPLICATION}/${ENV_CATEGORY}/${REGION}/tiller/ca.cert.pem" "$(helm home)/ca.pem"
fi

configured "helm version"

if [ -n "${K8S_NAMESPACE-}" ]; then
  kubectl config set-context --current --namespace="${K8S_NAMESPACE-}"
  echo "Set namespace to ${K8S_NAMESPACE-}"
fi

terraform version
configured "terraform version"

touch "${lock_file}"
