#!/usr/bin/env bash
set -euo pipefail
IFS=$'\n'

# For when script is called directly
if [ "${ENABLE_HELM3-false}" = "true" ]; then
  export PATH=/usr/local/bin/helm3/:$PATH
fi

# shellcheck source=src/bin/mosaic-common.bash
source "$(dirname "${BASH_SOURCE[0]}")/mosaic-common.bash"
set_usage "Mosaic helmfile Wrapper" "
Run helmfile against the \"helmfile.yaml\" file or \"helmfile.d\" directory in
the given directory.
"

function parse_helmfile_args {
  declare -ga helmfile_args

  if [ -n "${verbose}" ]; then
    helmfile_args=("--log-level=debug")
  fi
}

function helmfile_check {
  local directory="$1"

  (
    helmfile -version
    if [ "${ENABLE_HELM3-false}" = "true" ]
    then
      helm version --short
    else
      helm --tiller-namespace "$TILLER_NAMESPACE" version --short
    fi
    helm plugin list
    if_verbose cd "${directory}"
    KUBERNETES_CONTEXT="$(kubectl config current-context)" if_verbose helmfile "${helmfile_args[@]}" lint
  )
}

function helmfile_show {
  local directory="$1"
  local helmfile_nonglobal_args

  (
    helmfile -version
    helm version --short

    if [ -n "${CI_PIPELINE_SOURCE:-}" ] && [ "${CI_PIPELINE_SOURCE}" == "schedule" ]; then
      # shellcheck disable=SC2030,SC2031
      helmfile_nonglobal_args+=("--detailed-exitcode")
    fi

    echo "==> DEPRECATED APIs <=="
    echo "See https://confluence.carbonblack.local/display/MOSAIC/Upcoming+Kubernetes+API+Removals for more details about pluto"
    if [ "${ENABLE_HELM3-false}" = "true" ]
    then
      if_verbose pluto --kubeconfig ~/.kube/config detect-helm -n "$APPLICATION" -t k8s=v1.22.0 --ignore-deprecations --ignore-removals || true
    else
      if_verbose pluto --kubeconfig ~/.kube/config detect-helm --helm-version 2 --helm-store secrets -n "$TILLER_NAMESPACE" -t k8s=v1.18.0 --ignore-deprecations --ignore-removals || true
    fi
    if_verbose cd "${directory}"
    KUBERNETES_CONTEXT="$(kubectl config current-context)" if_verbose helmfile "${helmfile_args[@]}" diff --suppress-secrets "${helmfile_nonglobal_args[@]}"
  )
}

function helmfile_deploy {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    # shellcheck disable=SC2030,SC2031
    KUBERNETES_CONTEXT="$(kubectl config current-context)" if_verbose helmfile "${helmfile_args[@]}" sync

    if [ "${ENABLE_HELM_TEST:=true}" = "true" ]
    then
      if [ "${ENABLE_HELM3:=false}" = "true" ]
      then
        # shellcheck disable=SC2030,SC2031
        KUBERNETES_CONTEXT="$(kubectl config current-context)" if_verbose helmfile "${helmfile_args[@]}" test --args --logs
      else
        # shellcheck disable=SC2030,SC2031
        KUBERNETES_CONTEXT="$(kubectl config current-context)" if_verbose helmfile "${helmfile_args[@]}" test --cleanup
      fi
    fi
  )
}

function helmfile_destroy {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    # shellcheck disable=SC2030,SC2031
    KUBERNETES_CONTEXT="$(kubectl config current-context)" if_verbose helmfile "${helmfile_args[@]}" destroy
  )
}

mosaic-configure.bash
unset_AWS_env_creds
parse_args "$@"
parse_helmfile_args

case "${action}" in
  check)
    helmfile_check "${directory}"
    ;;
  show)
    ## Remove when helm2to3 migrations are done
    trap 'catch $?' EXIT
    # shellcheck source=src/bin/helm2to3.bash
    source "$(dirname "${BASH_SOURCE[0]}")/helm2to3.bash"
    if [ "${HELM2TO3-false}" = "true" ]
    then
      helm2to3_show
    else
      helmfile_show "${directory}"
    fi
    ##
    # helmfile_show "${directory}" ## Uncomment this line when helm2to3 migrations are done
    ;;
  deploy)
    ## Remove when helm2to3 migrations are done
    trap 'catch $?' EXIT
    # shellcheck source=src/bin/helm2to3.bash
    source "$(dirname "${BASH_SOURCE[0]}")/helm2to3.bash"
    if [ "${HELM2TO3-false}" = "true" ]
    then
      helm2to3_deploy
    else
      helmfile_deploy "${directory}"
    fi
    ##
    ## helmfile_deploy "${directory}" ## Uncomment this line when helm2to3 migrations are done
    ;;
  destroy)
    helmfile_destroy "${directory}"
    ;;
  *)
    usage
    ;;
esac
