#!/usr/bin/env bash
set -euo pipefail
IFS=$'\n'

if [ "${MOSAIC_USE_TF14-false}" = "true" ]; then
  export PATH=/usr/local/bin/terraform14/:$PATH
fi

# shellcheck source=src/bin/mosaic-common.bash
source "$(dirname "${BASH_SOURCE[0]}")/mosaic-common.bash"
set_usage "Mosaic terraform Wrapper" ""

function parse_tf_args {
  declare -ga tf_args

  if [ -n "${MOSAIC_TERRAFORM_VAR_FILE:-}" ]; then
    # Option format is a bit wierd, but this is the version that works for
    # both terraform and tflint.
    tf_args=("--var-file" "${MOSAIC_TERRAFORM_VAR_FILE}.tfvars")
  fi
}

function parse_tflint_args {
  declare -ga tflint_args
  tflint_args=("${tf_args[@]}")

  if [ -n "${MOSAIC_TFLINT_DEEP:-}" ]; then
    tflint_args+=("--deep")
  fi
}

function tf_command {
  if_verbose terraform init -input=false -upgrade=true

  if [ -n "${MOSAIC_TERRAFORM_WORKSPACE:-}" ]; then
    if_verbose terraform workspace new "${MOSAIC_TERRAFORM_WORKSPACE}" || \
      if_verbose terraform workspace select "${MOSAIC_TERRAFORM_WORKSPACE}"

    terraform init -input=false -upgrade=true
  fi

  if_verbose terraform "$@" "${tf_args[@]}"
}

function tf_check {
  local directory="$1"

  terraform version

  (
    if_verbose cd "${directory}"
    if_verbose terraform fmt -diff=true -check=true
    if_verbose terraform init -input=false -upgrade=true
    terraform validate
    tflint -v
    if_verbose tflint "${tflint_args[@]}" ./*.tf
  )
}

function tf_show {
  local directory="$1"

  (
    if_verbose cd "${directory}"

    # TF 0.13 Broke this functionality - will be fixed in 0.14
    # if [ -n "${CI_PIPELINE_SOURCE:-}" ] && [ "${CI_PIPELINE_SOURCE}" == "schedule" ]; then
    #   # shellcheck disable=SC2030
    #   tf_args+=("--detailed-exitcode")
    # fi

    # check if backend.tf exists. WARN for now.
    if [[ ! -f "backend.tf" ]]; then
      echo "no backend.tf found, exiting..."
      exit 1
    fi

    if [ -n "${MOSAIC_TERRAFORM_PLAN_FILE:-}" ]; then
      # shellcheck disable=SC2030
      tf_args+=("-out" "${MOSAIC_TERRAFORM_PLAN_FILE}")
    fi

    tf_command plan -input=false

    # Create JSON Terraform plan file if a plan is set
    if [ -n "${MOSAIC_TERRAFORM_PLAN_FILE:-}" ]; then
      terraform show -json "${MOSAIC_TERRAFORM_PLAN_FILE}" > "${MOSAIC_TERRAFORM_PLAN_FILE}.json"
    fi

    # TF 0.13 Workaround
    if [ -n "${CI_PIPELINE_SOURCE:-}" ] && [ "${CI_PIPELINE_SOURCE}" == "schedule" ] && [ -n "${MOSAIC_TERRAFORM_PLAN_FILE:-}" ]; then
      echo "Scheduled pipeline checking plan for changes, failing if changes found."
      terraform show -no-color "${MOSAIC_TERRAFORM_PLAN_FILE}" | grep -F "0 to add, 0 to change, 0 to destroy" > /dev/null
    fi
  )
}

function tf_deploy {
  local directory="$1"

  (
    if_verbose cd "${directory}"

    # check if backend.tf exists. WARN for now.
    if [[ ! -f "backend.tf" ]]; then
      echo "no backend.tf found, exiting..."
      exit 1
    fi

    if [ -n "${MOSAIC_TERRAFORM_PLAN_FILE:-}" ]; then
      if [ -s "${MOSAIC_TERRAFORM_PLAN_FILE}" ]; then
        tf_args=("${MOSAIC_TERRAFORM_PLAN_FILE}")
        tf_command apply -input=false -auto-approve
      else
        echo "Terraform plan file empty, skipping apply." >&2
      fi
    else
      tf_command apply -input=false -auto-approve
    fi

  )
}

function tf_destroy {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    tf_command destroy -input=false -auto-approve
  )
}

export CHECKPOINT_DISABLE=true
export TF_IN_AUTOMATION=true

mosaic-configure.bash
unset_AWS_env_creds
parse_args "$@"
parse_tf_args
parse_tflint_args

case "${action}" in
  check)
    tf_check "${directory}"
    ;;
  show)
    tf_show "${directory}"
    ;;
  deploy)
    tf_deploy "${directory}"
    ;;
  destroy)
    tf_destroy "${directory}"
    ;;
  *)
    usage
    ;;
esac
